@@ -22,11 +22,12 @@
 
 use crate::allocator_api::{AllocError, Allocator};
 use crate::internals::mem;
-use crate::util::{nonnull_as_mut_ptr, unlikely};
+use crate::util::{align_ptr_mut, nonnull_as_mut_ptr, unlikely};
 use crate::zeroize::{DefaultMemZeroizer, MemZeroizer};
 use core::alloc::Layout;
 use core::cell::Cell;
 use core::ptr::{self, NonNull};
+use mirai_annotations::debug_checked_precondition;
 
 /// Memory allocator for confidential memory. See the module level
 /// documentation.
@@ -152,7 +153,7 @@ impl<Z: MemZeroizer> Drop for SecStackSinglePageAlloc<Z> {
     }
 }
 
-#[cfg(unix)]
+#[cfg(any(unix, windows))]
 impl<Z: MemZeroizer> SecStackSinglePageAlloc<Z> {
     /// Create a new `SecStackSinglePageAlloc` allocator. This allocates one
     /// page of memory to be used by the allocator. This page is only
@@ -168,7 +169,7 @@ impl<Z: MemZeroizer> SecStackSinglePageAlloc<Z> {
     /// on Linux. A process with `CAP_SYS_RESOURCE` can change the `mlock`
     /// limit using `setrlimit` from libc.
     pub fn new_with_zeroizer(zeroizer: Z) -> Result<Self, mem::PageAllocError> {
-        let page = mem::Page::alloc_new_noreserve_mlock()?;
+        let page = mem::Page::alloc_new_lock()?;
         //let stack_ptr = page.page_ptr_nonnull();
         Ok(Self {
             zeroizer,
@@ -180,7 +181,7 @@ impl<Z: MemZeroizer> SecStackSinglePageAlloc<Z> {
     }
 }
 
-#[cfg(unix)]
+#[cfg(any(unix, windows))]
 impl<Z: MemZeroizer + Default> SecStackSinglePageAlloc<Z> {
     /// Create a new `SecStackSinglePageAlloc` allocator. This allocates one
     /// page of memory to be used by the allocator. This page is only
@@ -234,7 +235,7 @@ impl<Z: MemZeroizer> SecStackSinglePageAlloc<Z> {
     /// `align` must be a power of 2
     #[must_use]
     pub unsafe fn allocate_zerosized(align: usize) -> NonNull<[u8]> {
-        debug_assert!(align.is_power_of_two());
+        debug_checked_precondition!(align.is_power_of_two());
 
         // SAFETY: creating a pointer is safe, using it not; `dangling` is non-null
         let dangling: *mut u8 = align as *mut u8;
@@ -263,7 +264,7 @@ impl<Z: MemZeroizer> SecStackSinglePageAlloc<Z> {
     ) -> Result<NonNull<[u8]>, AllocError> {
         // like the default implementation of `Allocator::shrink` in the standard
         // library
-        debug_assert!(
+        debug_checked_precondition!(
             new_layout.size() <= old_layout.size(),
             "`new_layout.size()` must be smaller than or equal to `old_layout.size()`"
         );
@@ -303,7 +304,7 @@ impl<Z: MemZeroizer> SecStackSinglePageAlloc<Z> {
         new_layout: Layout,
     ) -> Result<NonNull<[u8]>, AllocError> {
         // like the default implementation of `Allocator::grow` in the standard library
-        debug_assert!(
+        debug_checked_precondition!(
             new_layout.size() >= old_layout.size(),
             "`new_layout.size()` must be greater than or equal to `old_layout.size()`"
         );
@@ -332,8 +333,7 @@ unsafe impl<Z: MemZeroizer> Allocator for SecStackSinglePageAlloc<Z> {
     // allows for fast zeroization and reduces the chance for (external) memory
     // fragmentation, at the cost of increased internal memory fragmentation.
     fn allocate_zeroed(&self, layout: Layout) -> Result<NonNull<[u8]>, AllocError> {
-        debug_assert!(layout.align() != 0); // implied by power of 2, but *very important* (safety)
-        debug_assert!(layout.align().is_power_of_two());
+        debug_checked_precondition!(layout.align().is_power_of_two());
 
         // catch zero sized allocations immediately so we do not have to bother with
         // them
@@ -388,25 +388,23 @@ unsafe impl<Z: MemZeroizer> Allocator for SecStackSinglePageAlloc<Z> {
             Ok(alloc_slice_ptr)
         } else {
             // slower path for large align
-            // subtract does not wrap since `layout.align()` is a power of 2, hence > 0
-            let align_minus_one = layout.align() - 1;
             // first pointer >= `stack_ptr` which is `layout.align()` bytes aligned
-            let next_aligned_ptr =
-                (stack_ptr as usize).wrapping_add(align_minus_one) & !align_minus_one;
-            // if this wraps the address space, then the result is 0 and the layout doesn't
-            // fit the remaining memory of our page, so error
-            if unlikely(next_aligned_ptr == 0) {
+            // SAFETY: `layout.align()` is a power of 2
+            let next_aligned_ptr = unsafe { align_ptr_mut(stack_ptr, layout.align()) };
+            // if this wraps the address space, then the result is null and the layout
+            // doesn't fit the remaining memory of our page, so error
+            if unlikely(next_aligned_ptr.is_null()) {
                 return Err(AllocError);
             }
             // offset of `next_align_ptr` relative from our base page pointer; doesn't wrap
             // since `next_align_ptr` is higher in the memory than `stack_ptr`
-            let next_align_pageoffset = next_aligned_ptr - (self.page.as_ptr() as usize);
+            let next_align_pageoffset = next_aligned_ptr as usize - (self.page.as_ptr() as usize);
             // error if `next_aligned_ptr` falls outside of our page
             if next_align_pageoffset >= self.page.page_size() {
                 return Err(AllocError);
             }
             // the new allocation will start at `next_aligned_ptr` and be `rounded_req_size`
-            // long error if we do not have enough space for this allocation
+            // long; error if we do not have enough space for this allocation
             // by the previous branch `self.page.page_size() - next_align_pageoffset` won't
             // wrap (`self.page.page_size() - next_align_pageoffset` is the
             // number of bytes available)
@@ -417,7 +415,7 @@ unsafe impl<Z: MemZeroizer> Allocator for SecStackSinglePageAlloc<Z> {
             // if we reach here then [next_aligned_ptr .. next_aligned_ptr +
             // rounded_req_size] lies entirely within our memory page
             let alloc_slice_ptr: *mut [u8] =
-                ptr::slice_from_raw_parts_mut(next_aligned_ptr as *mut u8, rounded_req_size);
+                ptr::slice_from_raw_parts_mut(next_aligned_ptr, rounded_req_size);
             // SAFETY: the page pointer is nonnull and the addition doesn't wrap so the
             // result is nonnull
             let alloc_slice_ptr: NonNull<[u8]> = unsafe { NonNull::new_unchecked(alloc_slice_ptr) };
@@ -449,8 +447,8 @@ unsafe impl<Z: MemZeroizer> Allocator for SecStackSinglePageAlloc<Z> {
 
         // `ptr` must be returned by this allocator, so it lies in the currently used
         // part of the memory page
-        debug_assert!(self.page.as_ptr() as usize <= ptr.as_ptr() as usize);
-        debug_assert!(
+        debug_checked_precondition!(self.page.as_ptr() as usize <= ptr.as_ptr() as usize);
+        debug_checked_precondition!(
             ptr.as_ptr() as usize <= self.page.as_ptr() as usize + self.stack_offset.get()
         );
 
@@ -503,7 +501,7 @@ unsafe impl<Z: MemZeroizer> Allocator for SecStackSinglePageAlloc<Z> {
         old_layout: Layout,
         new_layout: Layout,
     ) -> Result<NonNull<[u8]>, AllocError> {
-        debug_assert!(
+        debug_checked_precondition!(
             new_layout.size() <= old_layout.size(),
             "`new_layout.size()` must be smaller than or equal to `old_layout.size()`"
         );
@@ -522,8 +520,8 @@ unsafe impl<Z: MemZeroizer> Allocator for SecStackSinglePageAlloc<Z> {
 
         // `ptr` must be returned by this allocator, so it lies in the currently used
         // part of the memory page
-        debug_assert!(self.page.as_ptr() as usize <= ptr.as_ptr() as usize);
-        debug_assert!(
+        debug_checked_precondition!(self.page.as_ptr() as usize <= ptr.as_ptr() as usize);
+        debug_checked_precondition!(
             ptr.as_ptr() as usize <= self.page.as_ptr() as usize + self.stack_offset.get()
         );
 
@@ -597,7 +595,7 @@ unsafe impl<Z: MemZeroizer> Allocator for SecStackSinglePageAlloc<Z> {
         old_layout: Layout,
         new_layout: Layout,
     ) -> Result<NonNull<[u8]>, AllocError> {
-        debug_assert!(
+        debug_checked_precondition!(
             new_layout.size() >= old_layout.size(),
             "`new_layout.size()` must be greater than or equal to `old_layout.size()`"
         );
@@ -611,8 +609,8 @@ unsafe impl<Z: MemZeroizer> Allocator for SecStackSinglePageAlloc<Z> {
 
         // `ptr` must be returned by this allocator, so it lies in the currently used
         // part of the memory page
-        debug_assert!(self.page.as_ptr() as usize <= ptr.as_ptr() as usize);
-        debug_assert!(
+        debug_checked_precondition!(self.page.as_ptr() as usize <= ptr.as_ptr() as usize);
+        debug_checked_precondition!(
             ptr.as_ptr() as usize <= self.page.as_ptr() as usize + self.stack_offset.get()
         );
 
@@ -906,13 +904,13 @@ mod tests {
             let mut heap_mem = Vec::<u8, _>::with_capacity_in(9, &allocator);
             allocator.consistency_check();
             {
-                let heap_mem2 = Box::new_in(37_u64, &allocator);
+                let _heap_mem2 = Box::new_in(37_u64, &allocator);
                 allocator.consistency_check();
                 heap_mem.reserve(10);
                 allocator.consistency_check();
                 heap_mem.reserve(17);
                 allocator.consistency_check();
-            } // drop `heap_mem2`
+            } // drop `_heap_mem2`
             allocator.consistency_check();
         } // drop `heap_mem`
         allocator.consistency_check();
@@ -948,13 +946,13 @@ mod tests {
             let mut heap_mem = Vec::<u8, _>::with_capacity_in(9, &allocator);
             allocator.consistency_check();
             {
-                let heap_mem2 = Box::new_in(37_u64, &allocator);
+                let _heap_mem2 = Box::new_in(37_u64, &allocator);
                 allocator.consistency_check();
                 heap_mem.push(1);
                 allocator.consistency_check();
                 heap_mem.shrink_to_fit();
                 allocator.consistency_check();
-            } // drop `heap_mem2`
+            } // drop `_heap_mem2`
             allocator.consistency_check();
         } // drop `heap_mem`
         allocator.consistency_check();
